@@ -5,7 +5,6 @@ import click
 import time
 import os
 import re
-import copy
 from time import strftime, localtime
 from multiprocessing import Lock
 import traceback
@@ -27,9 +26,7 @@ class SQLExecute(object):
     SQLOptions = None                   # 程序处理参数
     Console = False                     # 屏幕输出Console
     logger = None                       # 日志输出
-
-    m_PerfFileLocker = Lock()           # 进程锁, 用来在输出perf文件的时候控制并发写文件
-
+    m_PerfFileLocker = None             # 进程锁, 用来在输出perf文件的时候控制并发写文件
     SQLPerfFile = None                  # SQLPerf文件
     SQLPerfFileHandle = None            # SQLPerf文件句柄
 
@@ -54,13 +51,28 @@ class SQLExecute(object):
         self.SQLTransaction = ''
 
     def jqparse(self, obj, path='.'):
+        class DecimalEncoder(json.JSONEncoder):
+            def default(self, o):
+                if isinstance(o, decimal.Decimal):
+                    return float(o)
+                super(DecimalEncoder, self).default(o)
+
+        if self is None:
+            pass
+        if obj is None:
+            if "SQLCLI_DEBUG" in os.environ:
+                click.secho("JQ Parse Error: obj is None")
+            return "****"
         try:
             obj = json.loads(obj) if isinstance(obj, str) else obj
             find_str, find_map = '', ['["%s"]', '[%s]', '%s', '.%s']
             for im in path.split('.'):
                 if not im:
                     continue
-
+                if obj is None:
+                    if "SQLCLI_DEBUG" in os.environ:
+                        click.secho("JQ Parse Error: obj is none")
+                    return "****"
                 if isinstance(obj, (list, tuple, str)):
                     if im.startswith('[') and im.endswith(']'):
                         im = im[1:-1]
@@ -73,7 +85,7 @@ class SQLExecute(object):
                         obj, quota = obj[int(im)], 1
                 else:
                     if im in obj:
-                        obj, quota = obj[im], 0
+                        obj, quota = obj.get(im), 0
                     elif im.endswith('()'):
                         obj, quota = list(getattr(obj, im[:-2])()), 3
                     else:
@@ -82,7 +94,10 @@ class SQLExecute(object):
                         else:
                             raise KeyError(im)
                 find_str += find_map[quota] % im
-            return obj if isinstance(obj, str) else json.dumps(obj,sort_keys=True,ensure_ascii=False)
+            return obj if isinstance(obj, str) else json.dumps(obj,
+                                                               sort_keys=True,
+                                                               ensure_ascii=False,
+                                                               cls=DecimalEncoder)
         except (IndexError, KeyError, ValueError) as je:
             if "SQLCLI_DEBUG" in os.environ:
                 click.secho("JQ Parse Error: " + repr(je))
@@ -117,7 +132,8 @@ class SQLExecute(object):
                 if self.SQLOptions.get("ECHO").upper() == 'ON' and self.logfile is not None:
                     click.echo(m_raw_sql, file=self.logfile)
                 if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
-                    click.echo(m_raw_sql, file=self.spoolfile)
+                    for m_SpoolFileHandler in self.spoolfile:
+                        click.echo(m_raw_sql, file=m_SpoolFileHandler)
                 if self.logger is not None:
                     if m_raw_sql is not None:
                         self.logger.info(m_raw_sql)
@@ -146,7 +162,8 @@ class SQLExecute(object):
                 # 在spool文件中，不显示spool off的信息，以避免log比对中的不必要内容
                 if not re.match(r'spool\s+.*', m_CommentSQL.strip(), re.IGNORECASE):
                     if self.SQLOptions.get("SILENT").upper() == 'OFF':
-                        click.echo(SQLFormatWithPrefix(m_CommentSQL), file=self.spoolfile)
+                        for m_SpoolFileHandler in self.spoolfile:
+                            click.echo(SQLFormatWithPrefix(m_CommentSQL), file=m_SpoolFileHandler)
 
             # 在logger中显示执行的SQL
             if self.logger is not None:
@@ -170,8 +187,9 @@ class SQLExecute(object):
                         click.echo(SQLFormatWithPrefix(
                             "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
                     if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
-                        click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
-                                   file=self.spoolfile)
+                        for m_SpoolFileHandler in self.spoolfile:
+                            click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
+                                       file=m_SpoolFileHandler)
                     if self.logger is not None:
                         self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
                     click.echo(SQLFormatWithPrefix(
@@ -200,8 +218,9 @@ class SQLExecute(object):
                     click.echo(SQLFormatWithPrefix(
                         "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
                 if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
-                    click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
-                               file=self.spoolfile)
+                    for m_SpoolFileHandler in self.spoolfile:
+                        click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
+                                   file=m_SpoolFileHandler)
                 if self.logger is not None:
                     self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
                 click.echo(SQLFormatWithPrefix(
@@ -239,8 +258,9 @@ class SQLExecute(object):
                     click.echo(SQLFormatWithPrefix(
                         "Your SQL has been changed to:\n" + sql, 'REWROTED '), file=self.logfile)
                 if self.SQLOptions.get("ECHO").upper() == 'ON' and self.spoolfile is not None:
-                    click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
-                               file=self.spoolfile)
+                    for m_SpoolFileHandler in self.spoolfile:
+                        click.echo(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '),
+                                   file=m_SpoolFileHandler)
                 if self.logger is not None:
                     self.logger.info(SQLFormatWithPrefix("Your SQL has been changed to:\n" + sql, 'REWROTED '))
                 click.echo(SQLFormatWithPrefix(
@@ -269,7 +289,15 @@ class SQLExecute(object):
                         if "SQLCLI_DEBUG" in os.environ:
                             click.secho("DEBUG-SQL=[" + str(sql) + "]", file=self.logfile)
                         # 执行SQL脚本
-                        cur.execute(sql)
+                        if "SQL_DIRECT" in m_SQLHint.keys():
+                            cur.execute_direct(sql)
+                        elif "SQL_PREPARE" in m_SQLHint.keys():
+                            cur.execute(sql)
+                        else:
+                            if self.SQLOptions.get("SQL_EXECUTE") == "DIRECT":
+                                cur.execute_direct(sql)
+                            else:
+                                cur.execute(sql)
                         rowcount = 0
                         while True:
                             (title, result, headers, columntypes, status, m_FetchStatus, m_FetchedRows) = \
@@ -292,7 +320,7 @@ class SQLExecute(object):
                                 # DEBUG模式下不会过滤任何东西
                                 pass
                             else:
-                                # 如果Hint中存在LogFilter，则过滤指定的输出信息
+                                # 如果Hint中存在LogFilter，则结果集中过滤指定的输出信息
                                 if "LogFilter" in m_SQLHint.keys() and result is not None:
                                     for m_SQLFilter in m_SQLHint["LogFilter"]:
                                         for item in result[:]:
@@ -300,7 +328,7 @@ class SQLExecute(object):
                                                 result.remove(item)
                                                 continue
 
-                                # 如果Hint中存在LogMask,，则掩码指定的输出信息
+                                # 如果Hint中存在LogMask,则掩码指定的输出信息
                                 if "LogMask" in m_SQLHint.keys() and result is not None:
                                     for i in range(0, len(result)):
                                         m_Output = None
@@ -310,6 +338,7 @@ class SQLExecute(object):
                                             else:
                                                 m_Output = m_Output + "," + str(result[i][j])
                                         for m_SQLMaskString in m_SQLHint["LogMask"]:
+                                            print("m_SQLMaskString=[" + m_SQLMaskString + "]")
                                             m_SQLMask = m_SQLMaskString.split("=>")
                                             if len(m_SQLMask) == 2:
                                                 m_SQLMaskPattern = m_SQLMask[0]
@@ -335,6 +364,7 @@ class SQLExecute(object):
                         for m_ErrorPrefix in ('ERROR:',):
                             if m_SQL_ErrorMessage.startswith(m_ErrorPrefix):
                                 m_SQL_ErrorMessage = m_SQL_ErrorMessage[len(m_ErrorPrefix):].strip()
+
                         # 发生了SQL语法错误
                         if self.SQLOptions.get("WHENEVER_SQLERROR") == "EXIT":
                             raise SQLCliException(m_SQL_ErrorMessage)
@@ -354,6 +384,42 @@ class SQLExecute(object):
                                               "com.microsoft.sqlserver.jdbc."):
                             if m_SQL_ErrorMessage.startswith(m_ErrorPrefix):
                                 m_SQL_ErrorMessage = m_SQL_ErrorMessage[len(m_ErrorPrefix):].strip()
+
+                        # 如果Hint中存在LogFilter，则输出的消息中过滤指定的输出信息
+                        if "LogFilter" in m_SQLHint.keys():
+                            m_SQL_MultiLineErrorMessage = m_SQL_ErrorMessage.split('\n')
+                            m_ErrorMessageHasChanged = False
+                            for m_SQLFilter in m_SQLHint["LogFilter"]:
+                                for item in m_SQL_MultiLineErrorMessage[:]:
+                                    if re.match(m_SQLFilter, ''.join(str(item)), re.IGNORECASE):
+                                        m_SQL_MultiLineErrorMessage.remove(item)
+                                        m_ErrorMessageHasChanged = True
+                                        continue
+                            if m_ErrorMessageHasChanged:
+                                m_SQL_ErrorMessage = "\n".join(m_SQL_MultiLineErrorMessage)
+
+                        # 如果Hint中存在LogMask,则掩码指定的输出信息
+                        if "LogMask" in m_SQLHint.keys():
+                            m_SQL_MultiLineErrorMessage = m_SQL_ErrorMessage.split('\n')
+                            m_ErrorMessageHasChanged = False
+                            for m_SQLMaskString in m_SQLHint["LogMask"]:
+                                m_SQLMask = m_SQLMaskString.split("=>")
+                                if len(m_SQLMask) == 2:
+                                    m_SQLMaskPattern = m_SQLMask[0]
+                                    m_SQLMaskTarget = m_SQLMask[1]
+                                    for m_nPos2 in range(0, len(m_SQL_MultiLineErrorMessage)):
+                                        m_NewOutput = re.sub(m_SQLMaskPattern, m_SQLMaskTarget,
+                                                             m_SQL_MultiLineErrorMessage[m_nPos2],
+                                                             re.IGNORECASE)
+                                        if m_NewOutput != m_SQL_MultiLineErrorMessage[m_nPos2]:
+                                            m_ErrorMessageHasChanged = True
+                                            m_SQL_MultiLineErrorMessage[m_nPos2] = m_NewOutput
+                                else:
+                                    if "SQLCLI_DEBUG" in os.environ:
+                                        raise SQLCliException("LogMask Hint Error: " + m_SQLHint["LogMask"])
+                            if m_ErrorMessageHasChanged:
+                                m_SQL_ErrorMessage = "\n".join(m_SQL_MultiLineErrorMessage)
+
                         # 发生了SQL语法错误
                         if self.SQLOptions.get("WHENEVER_SQLERROR") == "EXIT":
                             raise SQLCliException(m_SQL_ErrorMessage)
@@ -494,6 +560,10 @@ class SQLExecute(object):
         if self.SQLPerfFile is None:
             return
 
+        # 初始化文件加锁机制
+        if self.m_PerfFileLocker is None:
+            self.m_PerfFileLocker = Lock()
+
         # 多进程，多线程写入，考虑锁冲突
         try:
             self.m_PerfFileLocker.acquire()
